#import "fontawesome.typ": fa, globe, github, linkedin, envelope
#import "lib.typ": template, work_entry, project, indent, SMALL_STRONG, DEFAULT_STRONG

// orange!!
#let color = blue // rgb("#ff983e")
#let name = "Daniel Chen"

#let contact_data = ((
  service: fa[#globe],
  display: "eggworld.me",
  link: "https://eggworld.me",
), (
  service: fa[#github],
  display: "github.com/potatoeggy",
  link: "https://github.com/potatoeggy",
), (
  service: fa[#linkedin],
  display: "linkedin.com/in/potatoeggy",
  link: "https://www.linkedin.com/in/potatoeggy/",
), (
  service: fa[#envelope],
  display: "d7chen@uwaterloo.ca",
  link: "mailto://d7chen@uwaterloo.ca",
),)

#show: doc => template(name, contact_data, color, doc)

// MAIN CONTENT STARTS here

= Skills

#indent[
  *Languages:* Python, TypeScript, JavaScript, SQL, Rust, C++, Java, HTML, CSS,
  MATLAB \
  // Dart, Ruby
  *Frameworks:* React, Node.js, Spring Boot, Next.js, Vue, Flutter, Express,
  Prisma, Axum \
  // Qt, Jest / Pytest, Apollo, Prisma, Rails
  *Tools:* Linux, Git, Docker, PostgreSQL, AWS/GCP, Kubernetes, Terraform,
  GraphQL, nginx, OpenTelemetry // Jenkins, systemd, Jira
]

= Experience

#indent[
  #work_entry(
    "Technical Volunteer",
    "Organization for Transformative Works",
    start_date: "Feb 2024",
    end_date: "present",
    location: "Remote",
    tools: "Python, Ruby on Rails, React, MySQL",
    tasks: (
      [Upgrading archive import pipelines to migrate 4000+ user works to be accessible
        to *7 million registered users*],
    ),
  )

  #work_entry(
    "Backend Lead & Event Organiser",
    "Hack the North",
    start_date: "Mar 2023",
    end_date: "present",
    location: "Waterloo, ON",
    tools: "Express, GraphQL, Docker, Kubernetes",
    tasks: (
      [*Led and mentored a team of 6* to ideate, scope, and deliver *3 more major
        features* than the previous year],
      [Diagnosed and *recovered from load spike crashes* by debouncing queries, using
        rate limiting as a stopgap],
      [Reduced event check-in times by *80%* for *1900+ attendees* by generating
        digital wallet passes over Express],
      [Shipped a new *bus registration tool* for attendees by creating new database
        tables and GraphQL resolvers],
      [Integrated backend tracing to monitor performance and trigger alerts with
        OpenTelemetry and Grafana],
    ),
  )

  #work_entry(
    "Software Developer Intern",
    "Immigr8",
    start_date: "Jan 2024",
    end_date: "Apr 2024",
    location: "Remote",
    tools: "Next.js, Spring Boot, PostgreSQL, AWS, Terraform",
    tasks: (
      // [Improved client responsiveness by *80* by employing stale-while-revalidate hooks for caching and ?? SWR performance, caching, error handling],
      [*Introduced CI/CD for AWS Lambdas* by codifying them in Terraform for
        reproducible and automatic deployments],
      [Set up a staging environment via *EC2 and Jenkins* with parallel builds to
        *reduce production errors by 90%*],
      [Planned and executed a roadmap to SOC 2 security compliance by *implementing
        role-based permissions with JWTs*, sending backend logs and alerts to
        CloudWatch, and introducing new privacy controls for user data],
      [Reduced manual user inputs by *40%* by extracting data from users' uploaded
        documents in S3 with Textract],
    ),
  )

  #work_entry(
    "Software Engineering Intern",
    "Dropbase",
    start_date: "May 2023",
    end_date: "Aug 2023",
    location: "Toronto",
    tools: "React, Python, PostgreSQL, AWS",
    tasks: (
      [Designed a conflict resolution algorithm for merging databases, *reducing
        unrecoverable errors by 75%*],
      [Owned and launched a HubSpot/Airtable to Google Sheets sync feature to *46000+
        enterprise users* by writing AWS Lambda functions in Python, abstracting
        interfaces to easily support future sources],
      [Wrote and deployed a new React data query builder for our business intelligence
        visualisation service to *33000+ users*, eliminating the need for clients to
        manually write API calls or Python code],
      [Developed a *serverless microservice* that asynchronously ingests email
        attachments directly into ETL pipelines],
      // [??? emphasise test driven development or remove line *Reduced request errors by 30%* by creating backend testing infrastructure with
      //   *Pytest* to catch regressions and test new features across database engines with
      //   *90% code coverage*],
    ),
  )
]

= Projects
#set strong(delta: SMALL_STRONG)

#indent[
  #project(
    "Mandown Comic Downloader",
    "Python, Qt",
    repo_link: "https://github.com/potatoeggy/mandown",
  )
  - Created a CLI and GUI to scrape, download, and convert webcomics into
    e-ink-optimised ebooks (EPUB, PDF)
  - Designed a *scalable data conversion system* via a canonical format, allowing
    any format to convert to any other
  - Implemented *image caching and multithreading* to increase download and update
    speeds by *300%*

  // #project(
  //   "Genshin Simulator",
  //   "Vue, TypeScript",
  //   repo_link: "https://github.com/potatoeggy/primoprod",
  //   demo_link: "https://primoprod.vercel.app",
  // )
  // - Built a realistic and beautiful *loot box simulator* with CSS animations based
  //   off a popular game
  // - Reduced action latency by *90%* by lazily loading art assets at opportune times,
  //   such as during cutscenes

  #project(
    "Kobo Sync Server",
    "Rust, Axum",
    repo_link: "https://github.com/potatoeggy/kobink",
  )
  - *Reimplemented the Kobo Store Sync protocol* in an Axum backend to instead sync
    personal ebook files
  - Deployed the service behind an nginx reverse proxy and Cloudflare DNS to allow
    syncing over the internet

  // #project(
  //   "RecipeReady",
  //   "https://github.com/vincentjguo/recipeready-server",
  //   "Flutter, Selenium, FastAPI, Android",
  // )
  // - Developed a *Flutter app* to generate recipes based on ingredients and filters
  //   through a FastAPI server
  // - Scraped thousands of websites for recipes to build a database with a robust
  //   *many-to-many tag system*
  // - Aggregated common ingredients heuristically to build a shopping list and a meal
  // plan for each week

  // #project("Arch Linux Server", "Linux, Docker, nginx, systemd", demo_link: "https://eggworld.me")
  // - Created a *self-hosted CI/CD pipeline* with *Gitea* to automatically deploy
  //   personal projects behind *nginx*
  // - Containerised services with *Docker* and *systemd* to simplify deployment and
  //   management

  // #project(
  //   "Game Boy Advance Emulator",
  //   "Rust, SDL2, Vulkan",
  //   repo_link: "https://github.com/potatoeggy/gbarr",
  // )
  // - Implemented a *ARMv7 emulator* for old games to run on modern x86_64 platforms
  //   with *SDL2* and *Vulkan*
  // - WebGPU and WASM for portability, typestate to guarantee everything is always
  //   valid
]

= Education

#indent[
  #work_entry(
    "University of Waterloo",
    "B.A.Sc. in Computer Engineering",
    start_date: "Sep 2022",
    end_date: "Apr 2027",
    location: "Waterloo, ON",
    tasks: (
      [Courses: Object-oriented programming, data structures & algorithms, digital
        logic & circuits],
      // [Extracurriculars: Creative Writing Club, Hack the North, UW Orbital Design Team],
    ),
  )
]
